package org.artifactory.ph;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.aql.AqlConverts;
import org.artifactory.aql.AqlService;
import org.artifactory.aql.api.domain.sensitive.AqlApiItem;
import org.artifactory.aql.api.internal.AqlBase;
import org.artifactory.aql.model.AqlComparatorEnum;
import org.artifactory.aql.result.AqlEagerResult;
import org.artifactory.aql.result.rows.AqlItem;
import org.artifactory.core.api.ph.model.PackageArtifact;
import org.artifactory.core.api.ph.service.PackageSearchArtifactService;
import org.jetbrains.annotations.Nullable;
import org.jfrog.client.util.PathUtils;

import java.util.Map;
import java.util.stream.Stream;

import static org.artifactory.aql.api.domain.sensitive.AqlApiItem.property;

@Slf4j
public class PackageSearchArtifactServiceImpl implements PackageSearchArtifactService {

    public static final String PARAMETER = "Parameter '";
    private static final int UNLIMITED = -1;
    private final AqlService aqlService;
    private final AuthorizationService authorizationService;

    public PackageSearchArtifactServiceImpl(AqlService aqlService,
            AuthorizationService authorizationService) {
        this.aqlService = aqlService;
        this.authorizationService = authorizationService;
    }

    @Override
    public Stream<PackageArtifact> findArtifactsInExactPath(String repoKey, String path, String name) {
        name = StringUtils.isBlank(name) ? null : name.trim();
        path = StringUtils.isBlank(path) ? null : path.trim();
        assertRepoInput(repoKey);
        assertNoWildcards(path);
        assertNoMoreThanOneWildCard(name);
        // Creating the query with no extra fields
        AqlApiItem query = AqlApiItem.createWithEmptyResults();
        // Adding the fields we need
        buildIncludeQuery(query);
        // Preparing the query filters
        AqlBase.AndClause<AqlApiItem> and = AqlBase.and();
        and.append(AqlApiItem.repo().equal(repoKey));
        and.append(AqlApiItem.path().equal(path));
        filterByName(name, and);
        int length = PathUtils.getPathElements(path).length;
        and.append(AqlApiItem.depth().equals(length + 1));
        and.append(AqlApiItem.type().equal("file"));
        // Adding the AND filter to the query
        query.filter(and);
        return executeAqlQueryAndMapResults(query);
    }

    @Override
    public Stream<PackageArtifact> findChildrenArtifacts(String repoKey, String path, String name, int minDepth) {
        name = StringUtils.isBlank(name) ? null : name.trim();
        path = StringUtils.isBlank(path) ? null : path.trim();
        assertRepoInput(repoKey);
        assertNoMoreThanOneWildCard(path);
        assertWildCardAtTheEnd(path);
        assertNoMoreThanOneWildCard(name);
        // Creating the query with no extra fields
        AqlApiItem query = AqlApiItem.createWithEmptyResults();
        // Adding the fields we need
        buildIncludeQuery(query);
        // Preparing the query filters
        AqlBase.AndClause<AqlApiItem> and = AqlBase.and();
        and.append(AqlApiItem.repo().equal(repoKey));
        filterByPath(path, and);
        filterByName(name, and);
        and.append(AqlApiItem.depth().greaterEquals(minDepth));
        and.append(AqlApiItem.type().equal("file"));
        // Adding the AND filter to the query
        query.filter(and);
        return executeAqlQueryAndMapResults(query);
    }

    @Override
    public Stream<PackageArtifact> findArtifactsByProperties(String repoKey, String path,
            Map<String, String> properties) {
        return findArtifactsByPropertiesAndPaging(repoKey, path, properties, 0, UNLIMITED);
    }

    @Override
    public Stream<PackageArtifact> findArtifactsByPropertiesAndPaging(String repoKey, @Nullable String path,
            Map<String, String> properties,
            int offset, int limit) {
        assertRepoInput(repoKey);
        assertNoMoreThanOneWildCard(path);
        assertWildCardAtTheEnd(path);
        // Build filtering
        AqlBase.AndClause<AqlApiItem> and = AqlBase.and();
        //Add repo filter
        and.append(AqlApiItem.repo().equal(repoKey));
        //Add path filter
        filterByPath(path, and);
        //Add properties filter
        filterByProperties(properties, and);
        and.append(AqlApiItem.type().equal("file"));
        AqlApiItem query = AqlApiItem.create().filter(and);
        addOffsetAndLimitIfNeeded(query, offset, limit);
        return executeAqlQueryAndMapResults(query);
    }

    private void addOffsetAndLimitIfNeeded(AqlApiItem aql, int offset, int limit) {
        if (offset >= 0) {
            aql.offset(offset);
        }
        if (limit > 0) {
            aql.limit(limit);
        }
    }

    private void filterByProperties(Map<String, String> propertiesToMatch, AqlBase.AndClause<AqlApiItem> and) {
        for (Map.Entry<String, String> entry : propertiesToMatch.entrySet()) {
            String value = entry.getValue();
            if (value == null) {
                and.append(property().property(entry.getKey(), AqlComparatorEnum.equals, null));
            } else {
                assertWildCardAtTheEnd(value);
                and.append(property().property(
                        entry.getKey(),
                        value.contains("*") ? AqlComparatorEnum.matches : AqlComparatorEnum.equals,
                        value));
            }
        }
    }

    private void buildIncludeQuery(AqlApiItem query) {
        query.include(AqlApiItem.repo(), AqlApiItem.itemId(), AqlApiItem.path(), AqlApiItem.name(), AqlApiItem.type(),
                AqlApiItem.sha1Actual(), AqlApiItem.md5Actual(), AqlApiItem.sha2(), AqlApiItem.size(),
                AqlApiItem.created(), AqlApiItem.modified(), AqlApiItem.createdBy(), AqlApiItem.modifiedBy(),
                AqlApiItem.updated());
    }

    private void assertNoMoreThanOneWildCard(String param) {
        if (StringUtils.isNotEmpty(param) && StringUtils.countMatches(param, "*") >= 2) {
            throw new IllegalArgumentException(PARAMETER + param + "' cannot contain more than one '*'");
        }
    }

    private void assertWildCardAtTheEnd(String param) {
        int count = StringUtils.countMatches(param, "*");
        if (param != null && count >= 1 && !param.endsWith("*")) {
            throw new IllegalArgumentException(PARAMETER + param + "' can contain '*' only at the end");
        }
    }

    private void assertNoWildcards(String param) {
        if (StringUtils.isNotEmpty(param) && param.contains("*")) {
            throw new IllegalArgumentException(PARAMETER + param + "' cannot include '*' in exact match search");
        }
    }

    private void assertRepoInput(String repoKey) {
        if (StringUtils.isEmpty(repoKey)) {
            throw new IllegalArgumentException("RepoKey cannot be empty");
        }
    }

    /**
     * This method adds filters to the query in case filename has wildcards.
     * Take into consideration DB performance.
     *
     * @param fileName the file name to search for
     * @param and      the AND clause from the original AQL query
     */
    private void filterByName(String fileName, AqlBase.AndClause<AqlApiItem> and) {
        if (StringUtils.isNotBlank(fileName) && !fileName.equals("*")) {
            if (fileName.contains("*")) {
                and.append(AqlApiItem.name().matches(fileName));
                log.debug("Added AND clause to the artifact search query: AND node_name LIKE {}", fileName);
            } else {
                and.append(AqlApiItem.name().equal(fileName));
                log.debug("Added AND clause to the artifact search query: AND node_name = {}", fileName);
            }
        }
    }

    private void filterByPath(String path, AqlBase.AndClause<AqlApiItem> and) {
        if (StringUtils.isNotBlank(path) && !path.equals("*")) {
            if (path.contains("*")) {
                and.append(AqlApiItem.path().matches(path));
            } else {
                and.append(AqlApiItem.path().equal(path));
            }
        }
    }

    private Stream<PackageArtifact> executeAqlQueryAndMapResults(AqlApiItem query) {
        log.debug("Executing artifact search query");
        AqlEagerResult<AqlItem> allBlobs = aqlService.executeQueryEager(query);
        log.debug("Artifacts search completed");
        return allBlobs.getResults().stream()
                .map(AqlConverts.toFileInfo)
                .filter(file -> authorizationService.canRead(file.getRepoPath()))
                .map(PackageArtifactMapper::artifact);
    }
}
