package org.artifactory.servicetrust.federation;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.federated.FederatedRepoAddon;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.api.security.access.GenericTokenSpec;
import org.artifactory.api.security.access.TokenSpec;
import org.artifactory.descriptor.authentication.AuthenticationToken;
import org.artifactory.servicetrust.ExchangeTokenRequest;
import org.artifactory.servicetrust.ExchangeTokenResponse;
import org.artifactory.servicetrust.ServiceTrustUseCase;
import org.artifactory.servicetrust.TokenExtension;
import org.artifactory.servicetrust.TokenSpecInboundService;
import org.artifactory.servicetrust.TokenSpecOutboundService;
import org.artifactory.servicetrust.TokenSpecServiceBaseImpl;
import org.jetbrains.annotations.NotNull;
import org.jfrog.access.common.ServiceId;
import org.jfrog.access.token.JwtAccessToken;
import org.jfrog.common.JsonUtils;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static org.artifactory.servicetrust.ServiceTrustService.DEFAULT_EXCHANGE_TOKEN_EXPIRATION_TIME_IN_SECS;
import static org.artifactory.servicetrust.ServiceTrustService.INTERNAL_SCOPE_PREFIX;

@Slf4j
public class FederatedRepositoryTokenSpecService extends TokenSpecServiceBaseImpl
        implements TokenSpecOutboundService, TokenSpecInboundService {
    public static final String FEDERATED_BINARY_PROVIDER_USE_CASE = "$federated-binary-provider$";

    public static final FederatedRepositoryTokenSpecService FEDERATED_REPO_TOKEN_SPEC_SERVICE =
            new FederatedRepositoryTokenSpecService();
    public static final String PAIRING_TOKEN_FEDERATION_SCOPE_MISSING_ERROR =
            "Pairing token does not have federation related scope";
    public static final String PAIRING_TOKEN_SCOPES_NULL_OR_EMPTY_ERROR =
            "Pairing token scopes can not be null or empty";
    public static final String TOKEN_SCOPE_DOESNT_MATCH_FEDERATED_REPO_LEGAL_PATTERNS =
            "Token internal scope doesn't match federated-repo legal patterns";

    @NotNull
    @Override
    protected String getUseCasePathForScope(@NotNull String scope) {
        if (StringUtils.isEmpty(scope)) {
            throw new IllegalArgumentException(PAIRING_TOKEN_SCOPES_NULL_OR_EMPTY_ERROR);
        }
        return getUseCaseFromPairScope(Collections.singletonList(scope));
    }

    /**
     * Extract a use case with name 'federated-repo/*' from scopes
     */
    @Override
    @NotNull
    protected String getUseCaseFromPairScope(List<String> scopes) {

        String fedRepoScope = scopes
                .stream()
                .filter(s -> s.contains(ServiceTrustUseCase.FEDERATED_REPO.getName()))
                .findFirst()
                .orElseThrow(()->new IllegalArgumentException(PAIRING_TOKEN_FEDERATION_SCOPE_MISSING_ERROR));

        return getSimpleScope(fedRepoScope);
    }

    @NotNull
    @Override
    public String createRequiredInternalScope(@NotNull String useCasePath) {

        if(!ServiceTrustUseCase.FEDERATED_REPO.matches(useCasePath)) {
            throw new IllegalArgumentException(TOKEN_SCOPE_DOESNT_MATCH_FEDERATED_REPO_LEGAL_PATTERNS);
        }

        if(useCasePath.startsWith(INTERNAL_SCOPE_PREFIX))
            return useCasePath;

        return INTERNAL_SCOPE_PREFIX + useCasePath;
    }

    @Override
    public ExchangeTokenRequest createExchangeTokenRequest(String subject, JwtAccessToken jwt) {

        String scopedUseCase = getUseCaseFromPairScope(jwt.getScope());

        return new ExchangeTokenRequest(subject, scopedUseCase);
    }

    @Override
    public ExchangeTokenRequest createMasterTokenExchangeRefreshRequest(String subject) {
        return null;
    }

    @Override
    public void storeMasterTokenData(ExchangeTokenResponse response, TokenExtension extension) {
        getFederatedRepoAddon().saveAuthenticationMasterToken(response.getTokenId(), response.getTokenType(),
                response.getScope(), extension.getBaseUrl(), extension.getExchangeUrl(),
                response.getAccessToken(), response.getExpiresIn());
    }

    @Override
    public void updateMasterTokenData(ExchangeTokenResponse response) {
        // TODO BILAL 12: do we need to implement update here, unlike RT-on-ice federated token is always a new token.
    }

    @Override
    public ExchangeTokenRequest createScopedTokenExchangeRequest(String subject, String... scopes) {

        String target = Arrays.stream(scopes)
                .map(scp -> String.format("%s:x",
                        FederatedRepositoryTokenSpecService.FEDERATED_REPO_TOKEN_SPEC_SERVICE
                                .createRequiredInternalScope("federated-repo/" + scp)))
                .collect(Collectors.joining(" "));

        return new ExchangeTokenRequest(subject, target);
    }

    @Override
    public Collection<AuthenticationToken> getPairedInstances() {
        return getFederatedRepoAddon().getPairAuthenticationTokens();
    }

    @Override
    public TokenSpec<GenericTokenSpec> createExchangeTokenSpec(ServiceId artifactoryServiceId, String useCasePath) {

        String subject = artifactoryServiceId.getFormattedName();

        // This is a special token handling cases where binary-provider needs access to remote federation member.
        // Note: the use of $...$ is to avoid using exact repository name: 'federated-binary-provider'
        var scopes = useCasePath.contains(FEDERATED_BINARY_PROVIDER_USE_CASE)
                ? List.of("r,w,d,m".split(","))
                : List.of(createRequiredInternalScope(useCasePath));

        return GenericTokenSpec.create(subject)
                .refreshable(false)
                .audience(List.of(subject))
                .scope(scopes)
                .expiresIn(DEFAULT_EXCHANGE_TOKEN_EXPIRATION_TIME_IN_SECS)
                .extension(JsonUtils.getInstance().valueToString(TokenExtension.REVOCABLE_TOKEN));
    }

    @Override
    public void deleteAuthenticationToken(String baseUrl) {
        getFederatedRepoAddon().deleteAuthenticationToken(baseUrl);
    }

    @Override
    public Optional<AuthenticationToken> getAuthenticationToken(String baseUrl) {
        return getFederatedRepoAddon().getAuthenticationToken(baseUrl);
    }

    private FederatedRepoAddon getFederatedRepoAddon() {
        return ContextHelper.get().beanForType(AddonsManager.class).addonByType(FederatedRepoAddon.class);
    }
}
