package org.artifactory.event.work;

import org.artifactory.event.EventType;
import org.artifactory.event.priority.service.NodeEventPriorityService;
import org.artifactory.event.priority.service.model.PrioritizedNodeEvent;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.common.ClockUtils;
import org.jfrog.common.ExecutionUtils;
import org.jfrog.common.concurrency.ConflictsGuard;
import org.jfrog.metadata.client.exception.MetadataClientException;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoSession;
import org.mockito.quality.Strictness;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Collections;
import java.util.List;

import static org.artifactory.event.priority.service.NodeEventPriorityServiceImpl.MAX_RETRIES;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.anyString;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Uriah Levy
 */
@Test
public class NodeEventSweeperTest extends ArtifactoryHomeBoundTest {

    @Mock
    private NodeEventPriorityService priorityService;
    @Mock
    private NodeEventsErrorService nodeEventsErrorService;
    @Mock
    private NodeEventOperator operator;
    @Mock
    private EventOperatorDispatcher dispatcher;
    @Mock
    private ConflictsGuard<String> conflictsGuard;
    private MockitoSession mockitoSession;

    @BeforeMethod
    public void beforeMethod() {
        mockitoSession = Mockito.mockitoSession()
                .strictness(Strictness.STRICT_STUBS)
                .initMocks(this)
                .startMocking();
    }

    @AfterMethod
    public void afterMethod() {
        mockitoSession.finishMocking();
    }

    @Test
    public void testRun() {
        PrioritizedNodeEvent prioritizedNodeEvent = new PrioritizedNodeEvent(123L, "a/b/c", EventType.props,
                "metadata-event-operator", 0, ClockUtils.epochMillis(), 1, PrioritizedNodeEvent.EventStatus.PENDING);
        final List<PrioritizedNodeEvent> nodeEvents = Collections.singletonList(prioritizedNodeEvent);
        NodeEventSweeper nodeEventSweeper = spy(new NodeEventSweeper(priorityService, operator,
                dispatcher, nodeEvents, getRetryOptions(), nodeEventsErrorService, conflictsGuard));
        doNothing().when(nodeEventSweeper).tryToProcessEvent(nodeEvents.get(0));
        nodeEventSweeper.run();
        verify(dispatcher).sweeperDone();
        verify(nodeEventSweeper).tryToProcessEvent(nodeEvents.get(0));
        verify(nodeEventsErrorService, times(0))
                .process(any(String.class), any(PrioritizedNodeEvent.class), any(Exception.class));
    }

    @Test
    public void testRunWithProcessedEvent() {
        PrioritizedNodeEvent prioritizedNodeEvent = new PrioritizedNodeEvent(123L, "a/b/c", EventType.props,
                "metadata-event-operator", 0, ClockUtils.epochMillis(), 1, PrioritizedNodeEvent.EventStatus.PROCESSED);
        final List<PrioritizedNodeEvent> nodeEvents = Collections.singletonList(prioritizedNodeEvent);
        NodeEventSweeper nodeEventSweeper = spy(new NodeEventSweeper(priorityService, operator,
                dispatcher, nodeEvents, getRetryOptions(), nodeEventsErrorService, conflictsGuard));
        nodeEventSweeper.run();
        verify(dispatcher).sweeperDone();
        verify(nodeEventSweeper, times(0)).tryToProcessEvent(nodeEvents.get(0));
        verify(nodeEventsErrorService, times(0))
                .process(any(String.class), any(PrioritizedNodeEvent.class), any(Exception.class));
    }

    @Test
    public void testRetries() {
        PrioritizedNodeEvent prioritizedNodeEvent = new PrioritizedNodeEvent(123L, "a/b/c", EventType.props,
                "metadata-event-operator", 0, ClockUtils.epochMillis(), 0, PrioritizedNodeEvent.EventStatus.PENDING);
        final List<PrioritizedNodeEvent> nodeEvents = Collections.singletonList(prioritizedNodeEvent);

        when(operator.getEventCategory()).thenReturn("EventCategory");
        when(operator.isOperatorEnabled()).thenReturn(true);
        NodeEventSweeper nodeEventSweeper = spy(new NodeEventSweeper(priorityService, operator,
                dispatcher, nodeEvents, getRetryOptions(), nodeEventsErrorService, conflictsGuard));
        doAnswer(a -> {
            nodeEventSweeper.executeWithRetry(() -> nodeEventSweeper.doProcessEvent(prioritizedNodeEvent),
                    prioritizedNodeEvent);
            return null;
        }).when(dispatcher).runExclusive(any(), anyString(), any());
        MetadataClientException metadataClientException = new MetadataClientException("Metadata service error");
        doThrow(metadataClientException).when(nodeEventSweeper).doProcessEvent(prioritizedNodeEvent);
        nodeEventSweeper.run();
        verify(nodeEventSweeper, times(5)).doProcessEvent(prioritizedNodeEvent);
        verify(dispatcher, times(5)).batchError();
        verify(priorityService).saveRetryCount(any());
        verify(nodeEventsErrorService)
                .process("EventCategory", prioritizedNodeEvent, metadataClientException);
    }

    @Test
    public void testErrorRemovedAfterSuccess() {
        PrioritizedNodeEvent prioritizedNodeEvent = new PrioritizedNodeEvent(123L, "a/b/c", EventType.props,
                "metadata-event-operator", 0, ClockUtils.epochMillis(), 0, PrioritizedNodeEvent.EventStatus.PENDING);
        final List<PrioritizedNodeEvent> nodeEvents = Collections.singletonList(prioritizedNodeEvent);
        when(operator.getEventCategory()).thenReturn("EventCategory");
        when(operator.isOperatorEnabled()).thenReturn(true);
        NodeEventSweeper nodeEventSweeper = spy(new NodeEventSweeper(priorityService, operator,
                dispatcher, nodeEvents, getRetryOptions(), nodeEventsErrorService, conflictsGuard));
        doAnswer(a -> {
            nodeEventSweeper.executeWithRetry(() -> nodeEventSweeper.doProcessEvent(prioritizedNodeEvent),
                    prioritizedNodeEvent);
            return null;
        }).when(dispatcher).runExclusive(any(), anyString(), any());
        nodeEventSweeper.run();
        verify(nodeEventSweeper, times(1)).doProcessEvent(prioritizedNodeEvent);
        verify((nodeEventsErrorService), times(1)).removeError("EventCategory", prioritizedNodeEvent);
    }

    private ExecutionUtils.RetryOptions getRetryOptions() {
        return ExecutionUtils.RetryOptions.builder()
                .backoffMaxDelay(5)
                .exponentialBackoffMultiplier(3)
                .numberOfRetries(MAX_RETRIES)
                .timeout(10)
                .build();
    }
}