package org.artifactory.projects;

import org.artifactory.aol.AolService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Set;

import static org.artifactory.repo.system.SystemRepos.BUILDINFO_REPO_KEY;
import static org.mockito.Mockito.clearInvocations;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * @author BarakH
 */
public class ProjectsUtilsTest {

    @Mock
    private ArtifactoryContext artifactoryContext;

    private AutoCloseable autoCloseable;

    @BeforeMethod
    void setup() {
        autoCloseable = MockitoAnnotations.openMocks(this);
        ArtifactoryContextThreadBinder.bind(artifactoryContext);
    }

    @AfterMethod
    void teardown() throws Exception {
        ArtifactoryContextThreadBinder.unbind();
        autoCloseable.close();
    }

    @Test(dataProvider = "provideProjectNames")
    public void testIsDefaultProject(String project, boolean isDefault) {
        boolean result = ProjectsUtils.isDefaultProject(project);
        Assert.assertEquals(result, isDefault);
    }

    @Test(dataProvider = "provideProjectNameAndExpectError")
    public void testAssertVirtualRepositoriesAreInProject(String projectName, String expectError) {
        ProjectsService projectsService = mock(ProjectsService.class);
        when(projectsService.getProjectRepositoriesKeys("project1", false))
                .thenReturn(Set.of("A", "B"));
        when(projectsService.getProjectRepositoriesKeys("project2", false))
                .thenReturn(Set.of("C"));
        String error = "";
        try {
            ProjectsUtils.assertVirtualRepositoriesAreInProject(projectsService, projectName, List.of("A"));
        } catch (IllegalArgumentException e) {
            error = e.getMessage();
        }
        Assert.assertEquals(error, expectError);
    }

    @Test
    public void testGetProjectKeyFromRepoKey() {
        Assert.assertEquals(ProjectsUtils.getProjectKeyFromRepoKey(BUILDINFO_REPO_KEY),
                ProjectsService.DEFAULT_PROJECT);
        Assert.assertEquals(ProjectsUtils.getProjectKeyFromRepoKey("prjx-build-info"), "prjx");
        Assert.assertEquals(ProjectsUtils.getProjectKeyFromRepoKey("prjx-build"), "prjx");
        Assert.assertNull(ProjectsUtils.getProjectKeyFromRepoKey("prjx"));
        Assert.assertNull(ProjectsUtils.getProjectKeyFromRepoKey(""));
    }

    @Test
    public void testIsStorageQuotaSoftLimit() {
        AolService aolService = mock(AolService.class);
        when(artifactoryContext.beanForType(AolService.class)).thenReturn(aolService);
        when(aolService.isAol()).thenReturn(true);

        assertTrue(ProjectsUtils.isStorageQuotaSoftLimit(false));
        assertTrue(ProjectsUtils.isStorageQuotaSoftLimit(true));
        verify(aolService).isAol();
        clearInvocations(aolService);

        when(aolService.isAol()).thenReturn(false);
        assertFalse(ProjectsUtils.isStorageQuotaSoftLimit(false));
        assertTrue(ProjectsUtils.isStorageQuotaSoftLimit(true));
        verify(aolService).isAol();
    }

    @DataProvider
    public static Object[][] provideProjectNames() {
        return new Object[][] {
                {null, true},
                {"", true},
                {ProjectsService.DEFAULT_PROJECT, true},
                {"DEFAULT", true},
                {"project", false},
                {"default-project", false},
        };
    }

    @DataProvider
    public static Object[][] provideProjectNameAndExpectError() {
        return new Object[][] {
                {null, ""},
                {"", ""},
                {ProjectsService.DEFAULT_PROJECT, ""},
                {"project1", ""},
                {"project2", "Virtual repository must include only repositories from the same project it was created " +
                        "with"},
        };
    }
}
