package org.artifactory.release.bundle;

import org.artifactory.api.common.BasicStatusHolder;
import org.artifactory.api.rest.distribution.bundle.models.ArtifactsBundleModel;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.bundle.BundleTransactionStatus;
import org.artifactory.bundle.BundleType;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.repo.service.InternalRepositoryService;
import org.artifactory.storage.db.bundle.dao.ArtifactBundlesDao;
import org.artifactory.storage.db.bundle.dao.BundleBlobsDao;
import org.artifactory.storage.db.bundle.model.BundleNode;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.sql.SQLException;
import java.util.List;
import java.util.Set;

import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Eli Skoran 02/09/2020
 */

@Test
public class InternalReleaseBundleServiceTest {

    @Mock
    private ArtifactBundlesDao bundlesDao;
    @Mock
    private BundleBlobsDao blobsDao;
    @Mock
    private InternalRepositoryService repoService;
    @Mock
    private AuthorizationService authService;
    @Mock
    private ReleaseBundleDeleteTemplate releaseBundleDeleteTemplate;

    private InternalReleaseBundleService service;

    private InternalReleaseBundleService innerService; // Two mock services defined because of circular dependency

    @BeforeMethod
    public void setup() throws SQLException {
        MockitoAnnotations.initMocks(this);
        innerService = new InternalReleaseBundleServiceImpl(
                bundlesDao,
                blobsDao,
                repoService,
                authService,
                mock(ReleaseBundleDeleteTemplate.class)
        );
    }

    @AfterMethod
    public void tearDown() {
        Mockito.verifyNoMoreInteractions(
                bundlesDao,
                blobsDao,
                repoService,
                authService,
                releaseBundleDeleteTemplate
        );

    }

    @Test
    public void testDeleteReleaseBundle_Complete() throws SQLException {
        service = new InternalReleaseBundleServiceImpl(
                bundlesDao,
                blobsDao,
                repoService,
                authService,
                new GeneralReleaseBundleDeleteTemplate(bundlesDao, innerService)
        );
        // setup
        ArtifactsBundleModel bundleModel = new ArtifactsBundleModel(
                111,
                "bundle1",
                "version1",
                BundleTransactionStatus.COMPLETE,
                BundleType.SOURCE,
                "release_bundles"
        );
        List<BundleNode> bundleNodes = List.of(
                new BundleNode(1234, 234, "path/to/repo", 111, "originalDetails"),
                new BundleNode(1235, 235, "another/path/to/repo", 111, "originalDetails")
        );

        RepoPath firstRepoPath = RepoPathFactory.create("path/to/repo");
        RepoPath secondRepoPath = RepoPathFactory.create("another/path/to/repo");

        RepoPath exportedFileRepoPath =
                ReleaseBundleUtils.buildExportFileRepoPath("bundle1", "version1", "release_bundles");

        Set<Long> nodeIdsWithArtifactsToDelete = Set.of(1234L, 1235L);

        doNothing().when(bundlesDao).deleteBundleNodes(bundleModel.getId());
        when(blobsDao.deleteBlob(bundleModel.getId())).thenReturn(true);
        when(bundlesDao.deleteArtifactsBundle(bundleModel.getId())).thenReturn(true);

        when(repoService.undeploy(firstRepoPath)).thenReturn(new BasicStatusHolder());
        when(repoService.undeploy(secondRepoPath)).thenReturn(new BasicStatusHolder());

        when(repoService.exists(exportedFileRepoPath)).thenReturn(true);
        when(repoService.undeploy(exportedFileRepoPath)).thenReturn(new BasicStatusHolder());

        // test
        service.deleteBundleInternal(bundleModel, bundleNodes, nodeIdsWithArtifactsToDelete);

        // verify
        verify(bundlesDao).deleteBundleNodes(bundleModel.getId());
        verify(blobsDao).deleteBlob(bundleModel.getId());
        verify(bundlesDao).deleteArtifactsBundle(bundleModel.getId());

        verify(repoService).undeploy(firstRepoPath);
        verify(repoService).undeploy(secondRepoPath);

        verify(repoService).exists(exportedFileRepoPath);
        verify(repoService).undeploy(exportedFileRepoPath);
    }

    @Test
    public void testDeleteReleaseBundle_NotExported() throws SQLException {
        service = new InternalReleaseBundleServiceImpl(
                bundlesDao,
                blobsDao,
                repoService,
                authService,
                new GeneralReleaseBundleDeleteTemplate(bundlesDao, innerService)
        );
        // setup
        ArtifactsBundleModel bundleModel = new ArtifactsBundleModel(
                111,
                "bundle1",
                "version1",
                BundleTransactionStatus.COMPLETE,
                BundleType.SOURCE,
                "release_bundles"
        );
        List<BundleNode> bundleNodes = List.of(
                new BundleNode(1234, 234, "path/to/repo", 111, "originalDetails"),
                new BundleNode(1235, 235, "another/path/to/repo", 111, "originalDetails")
        );

        RepoPath firstRepoPath = RepoPathFactory.create("path/to/repo");
        RepoPath secondRepoPath = RepoPathFactory.create("another/path/to/repo");

        RepoPath exportedFileRepoPath =
                ReleaseBundleUtils.buildExportFileRepoPath("bundle1", "version1", "release_bundles");

        Set<Long> nodeIdsWithArtifactsToDelete = Set.of(1234L, 1235L);

        doNothing().when(bundlesDao).deleteBundleNodes(bundleModel.getId());
        when(blobsDao.deleteBlob(bundleModel.getId())).thenReturn(true);
        when(bundlesDao.deleteArtifactsBundle(bundleModel.getId())).thenReturn(true);

        when(repoService.undeploy(firstRepoPath)).thenReturn(new BasicStatusHolder());
        when(repoService.undeploy(secondRepoPath)).thenReturn(new BasicStatusHolder());

        when(repoService.exists(exportedFileRepoPath)).thenReturn(false);

        // test
        service.deleteBundleInternal(bundleModel, bundleNodes, nodeIdsWithArtifactsToDelete);

        // verify
        verify(bundlesDao).deleteBundleNodes(bundleModel.getId());
        verify(blobsDao).deleteBlob(bundleModel.getId());
        verify(bundlesDao).deleteArtifactsBundle(bundleModel.getId());

        verify(repoService).undeploy(firstRepoPath);
        verify(repoService).undeploy(secondRepoPath);

        verify(repoService).exists(exportedFileRepoPath);
    }
}
