package org.artifactory.core.api.ph.virtual.resolution;

import org.artifactory.common.resource.BaseResource;
import org.artifactory.core.api.ph.exception.PackageVirtualTraversalException;
import org.jetbrains.annotations.NotNull;

import java.io.InputStream;
import java.util.List;
import java.util.Map;

/**
 * Virtual repo merging controller API
 * in order to merge the artifacts metadata, the virtual repo iterates over all repositories
 */
public interface PackageVirtualMerger<T extends PackageMetadataRow> {

    /**
     * This method is called by the virtual visitor (in case of merging) framework.
     * The virtual merger framework needs it deserialize specific metadata stream into metadata object
     * The method needs in order that it will be able to collect al the metadata object from the various
     * metadata artifacts in the sub-repositories
     *
     * The method is not allowed to return null, but it can return empty list
     *
     * @param res            File info resource found: The resource must be available (found + not blocked +not expired)
     * @param virtualRepoKey The virtual repo key
     * @param originalPath   The virtual repo path (translated)
     * @return Extracted metadata in list of rows
     */
    @NotNull
    Map<String, List<T>> extract(BaseResource res, String virtualRepoKey, String originalPath) throws
            PackageVirtualTraversalException;

    /**
     * This method is called by the virtual visitor (in case of merging).
     * The virtual merger framework needs it transform the collected metadata into single Into input stream
     * in order that the virtual merger framework will be able to persist it as an artifact
     *
     * The method is being called only if metadata is not empty therefore it should never return null
     *
     * @param metadata       aggregated metadata
     * @param requestContext original request context
     * @return Input stream in order that the virtual merger framework will be able to persist it as an artifact
     */
    @NotNull
    InputStream toInputStream(Map<String, PackageMetadataVersionsList<T>> metadata,
            PackageRequestContext requestContext) throws PackageVirtualTraversalException;

    /**
     * This method is called by the virtual visitor (in case of merging).
     * The virtual merger use it to check if it should use the cache for the specific artifact
     *
     * Note this method is used (called) only in case of merging process.
     * In case of normal resolution we should never use cache
     *
     * @param virtualRepoKey the artifact repo key
     * @param path           the artifact path
     * @return true in case should use virtual cache and false in case that caching merged metadata is not allowed
     * @throws PackageVirtualTraversalException in case of internal error
     */
    default boolean shouldCacheMergedMetadata(String virtualRepoKey, String path) throws
            PackageVirtualTraversalException {
        return true;
    }
}