#!/usr/bin/env bash

set -e

export PROJECT_ROOT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && cd ../../.. && pwd)"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/commonUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/mavenUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/dockerUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/versionUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/pipelineUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/buildInfoUtils.sh"

function on_success() {
  write_BuildInfo
  export JFROG_CLI_BUILD_NUMBER="${BUILD_NUMBER}"
  export JFROG_CLI_BUILD_NAME="${ART_OSS_BUILD_NAME}"

  buildInfo_modifyBuildName "${JFROG_CLI_BUILD_NAME}" "${JFROG_CLI_BUILD_NUMBER}"
  buildInfo_publish "${JFROG_CLI_BUILD_NAME}"

  if [[ "${INSTALLER_OSS_PROFILE_TYPE}" == "docker" ]]; then
      # Push and publish a build with the docker images
      pushAndUpdateResource "jfrog/artifactory-oss" "artifactory_oss_art_jfrog_io_docker_image" "$DOCKER_IMAGE_VERSION" "$DOCKER_PUSH_REPO" "$DOCKER_PUSH_REGISTRY" "$ART_OSS_DOCKER_BUILD_NAME"
      buildInfo_publish "${ART_OSS_DOCKER_BUILD_NAME}"
  fi

  ## Trigger relevant installer types in parallel
  if [[ "${INSTALLER_OSS_PROFILE_TYPE}" == "release" ]]; then
      write_output "artifactory_oss_build_installer_docker" proBuildDetails="${JFROG_CLI_BUILD_NAME}-${JFROG_CLI_BUILD_NUMBER}"
      if [[ "${INCLUDE_ALL_INSTALLER_TYPES}" == "true" ]]; then
        write_output "artifactory_oss_build_installer_rpm" proBuildDetails="${JFROG_CLI_BUILD_NAME}-${JFROG_CLI_BUILD_NUMBER}"
        write_output "artifactory_oss_build_installer_deb" proBuildDetails="${JFROG_CLI_BUILD_NAME}-${JFROG_CLI_BUILD_NUMBER}"
      fi
  fi
  return 0
}

function on_complete() {
  docker_cleanupAllUnusedContainersAndImages || echo "Docker images cleanup finished with an error"
  return 0
}

function trigger_installer_package() {
  if ! common_retry "trigger installers" 5 "__call_installer_package_hook $PRODUCT_DEPLOY_REPO"; then
    echo "[ERROR] Failed to trigger installer package pipeline." >&2
  fi
}

function __call_installer_package_hook(){
  curl -X POST "${int_installer_bundler_pipeline_hook_url}" -d \
    '{
        "PRODUCT_RESOLVE_REPO":"'${PRODUCT_DEPLOY_REPO}'",
        "PRODUCT_DEPLOY_REPO":"'${PRODUCT_DEPLOY_REPO}'",
        "PRODUCT_VERSION":"'${PRODUCT_VERSION}'",
        "PRODUCT_NAME":"artifactory_oss",
        "BUILD_NUMBER":"'${JFROG_CLI_BUILD_NUMBER}'",
        "DOCKER_PUSH_REPO": "'${DOCKER_PUSH_REPO}'",
        "INSTALLER_COMMON_REPO":"'${PRODUCT_DEPLOY_REPO}'",
        "INSTALLER_COMMON_VERSION":"'${INSTALLER_COMMON_VERSION}'",
        "DOCKER_PUSH_REGISTRY": "'${DOCKER_PUSH_REGISTRY}'",
        "ART_OSS_DOCKER_BUILD_NAME": "'${ART_OSS_DOCKER_BUILD_NAME}'",
        "DOCKER_IMAGE_VERSION" : "'${DOCKER_IMAGE_VERSION}'",
        "ART_OSS_BUILD_NAME":"'${ART_OSS_INSTALLERS_BUILD_NAME}'",
        "INSTALLER_TYPE":"'${INSTALLER_PACKAGE_TYPES}'"
    }' \
    -u "${int_installer_bundler_pipeline_hook_username}:${int_installer_bundler_pipeline_hook_password}" \
    -H 'Content-Type: application/json' \
    --silent --fail --show-error
}

function on_start() {
  cd "${PROJECT_ROOT_DIR}/artifactory-oss"
  # Publish the build with the Maven artifacts
  local version
  version="$(maven_getVersionFromPom "${PROJECT_ROOT_DIR}/artifactory-oss/pom.xml")"
  buildInfo_setBuildNameAndNumber
  setProfileType
  local branch
  local normalizedBranch
  local repo
  echo "Calculating TARGET_VERSION and DOCKER_COMPOSE_REGISTRY for the docker compose env"
  echo "Version: ${version}"
  branch="$(pipeline_gitBranchName)"
  echo "Branch:  ${branch}"
  normalizedBranch=$(common_normalizeBranchName "${branch}")

  local cache_dir="${run_dir}/${step_affinity_group}_cache_classes"
  local state_cache_dir="${PROJECT_ROOT_DIR}/artifactory-oss"
  local run_state_cache_dir="${cache_dir}/dir_cache"
  if cp -r "${run_state_cache_dir}"/* "${state_cache_dir}"; then
    echo "Dir state was restored." >&2
  fi

  local targetVersion="${version}-${normalizedBranch}-${run_number}"
  local dockerComposeRegistry="docker.jfrog.io"
  if version_isRelease "${version}" || version_isMilestone "${version}"; then
    targetVersion="${version}"
    if version_isRelease "${version}"; then
      dockerComposeRegistry="releases-docker.jfrog.io"
    fi
  fi
  
  if common_isReleaseOrMilestoneBranch "${branch}"; then
    PRODUCT_TIMESTAMP=$(common_getTimestampFromReleaseBranchName "${branch}")
  else
    PRODUCT_TIMESTAMP="LOCAL"
  fi
  
  PRODUCT_REVISION=$(version_getRevisionFromVersion ${version})

  export TARGET_VERSION="${targetVersion}" DOCKER_COMPOSE_REGISTRY="$dockerComposeRegistry"

  # Push and publish a build with the docker images
  local repository="art-docker-dev-local"
  local imageName="jfrog/artifactory-oss"
  local dockerVirtualRegistry="docker.jfrog.io"

  PRODUCT_DEPLOY_REPO="$(maven_devArtifactRepoName "$branch")"

  PRODUCT_VERSION="$version"
  DOCKER_IMAGE_VERSION="$targetVersion"
  DOCKER_PUSH_REPO="${repository}"
  DOCKER_PUSH_REGISTRY="art.jfrog.io"

  setCustomBuildTypes
  ART_OSS_INSTALLERS_BUILD_NAME="${JFROG_CLI_BUILD_NAME}"
  ART_OSS_DOCKER_BUILD_NAME="$(docker_generateBuildName)"

  if [[ $(common_gitLastCommitBody) == *"[INSTALLER_DEV]"* ]] || [[ "${INSTALLER_DEV}" == true ]]; then
    INSTALLER_COMMON_REPO="$(common_devArtifactRepoName "$branch" "maven" "local")"
    INSTALLER_COMMON_VERSION="7*"
  else
    INSTALLER_COMMON_REPO="libs-staging"
    INSTALLER_COMMON_VERSION=$(maven_getPropertyValueFromPom "jfrog.installer.commons.version" "${PROJECT_ROOT_DIR}/artifactory-oss/pom.xml")
  fi

  export INSTALLER_PACKAGE_TYPES="${INSTALLER_OSS_PROFILE_TYPE}"

  export BUILD_PARAMS_JSON="${res_artifactory_oss_bitbucket_resourcePath}/installers/src/products/build-params.json"

  echo '{
        "PRODUCT_RESOLVE_REPO":"'${PRODUCT_DEPLOY_REPO}'",
        "PRODUCT_DEPLOY_REPO":"'${PRODUCT_DEPLOY_REPO}'",
        "PRODUCT_VERSION":"'${PRODUCT_VERSION}'",
        "PRODUCT_NAME":"artifactory_oss",
        "BUILD_NUMBER":"'${JFROG_CLI_BUILD_NUMBER}'",
        "DOCKER_PUSH_REPO": "'${DOCKER_PUSH_REPO}'",
        "INSTALLER_COMMON_REPO":"'${INSTALLER_COMMON_REPO}'",
        "INSTALLER_COMMON_VERSION":"'${INSTALLER_COMMON_VERSION}'",
        "DOCKER_PUSH_REGISTRY": "'${DOCKER_PUSH_REGISTRY}'",
        "ART_OSS_DOCKER_BUILD_NAME": "'${ART_OSS_DOCKER_BUILD_NAME}'",
        "DOCKER_IMAGE_VERSION" : "'${DOCKER_IMAGE_VERSION}'",
        "ART_OSS_BUILD_NAME":"'${ART_OSS_INSTALLERS_BUILD_NAME}'",
        "INSTALLER_TYPE":"'${INSTALLER_PACKAGE_TYPES}'",
        "PRODUCT_DISABLE_PROFILES":"'${PRODUCT_DISABLE_PROFILES}'",
        "PRODUCT_REVISION":"'${PRODUCT_REVISION}'",
        "PRODUCT_TIMESTAMP":"'${PRODUCT_TIMESTAMP}'"
    }' | jq '.' > "${BUILD_PARAMS_JSON}"

  #download installer-commons from branch snapshot
  ${res_artifactory_oss_bitbucket_resourcePath}/installers/src/products/download.sh --user ${int_entplus_deployer_user} --pass ${int_entplus_deployer_apikey}
  source ${res_artifactory_oss_bitbucket_resourcePath}/installers/src/products/build-src/configuration-manager/buildUtils/commonUtils.sh
  exportJsonParams "${BUILD_PARAMS_JSON}"

  cd ${res_artifactory_oss_bitbucket_resourcePath}/installers/src/products/build-src/configuration-manager/products/artifactory/artifactory-oss/distribution
  source ${res_artifactory_oss_bitbucket_resourcePath}/installers/src/products/build-src/configuration-manager/products/artifactory/artifactory-oss/package.sh
  source ${res_artifactory_oss_bitbucket_resourcePath}/installers/src/products/build-src/configuration-manager/products/pack_env.sh
  exportEnvAsRunVariables "${RUN_VARIABLE_KEYLIST}"

  prepare_maven_config
}

function write_BuildInfo () {
    case "${INSTALLER_OSS_PROFILE_TYPE}" in
        docker )
            write_output "artifactory_oss_art_jfrog_io_docker_image" sourceRepository="${repository}" imageName="${dockerVirtualRegistry}/${imageName}" imageTag="${targetVersion}"
            write_output artifactory_oss_docker_build_info buildName="${ART_OSS_DOCKER_BUILD_NAME}" buildNumber="${JFROG_CLI_BUILD_NUMBER}"
            write_output artifactory_oss_installers_compose_build_info buildName="${ART_OSS_INSTALLERS_BUILD_NAME}" buildNumber="${JFROG_CLI_BUILD_NUMBER}"
            ;;
        deb )
            write_output artifactory_oss_installers_deb_build_info buildName="${JFROG_CLI_BUILD_NAME}" buildNumber="${JFROG_CLI_BUILD_NUMBER}" version="${targetVersion}"
            ;;
        rpm )
            write_output artifactory_oss_installers_rpm_build_info buildName="${JFROG_CLI_BUILD_NAME}" buildNumber="${JFROG_CLI_BUILD_NUMBER}" version="${targetVersion}"
            ;;
        * )
            write_output artifactory_oss_package_installers_build_info buildName="${ART_OSS_INSTALLERS_BUILD_NAME}" buildNumber="${JFROG_CLI_BUILD_NUMBER}" version="${targetVersion}"
            ;;
    esac
}

# exports an environment variable with desired build types based on the last git commit body or env variable
function setCustomBuildTypes(){
  local branch
  branch="$(pipeline_gitBranchName)"
  if [[ $(common_gitLastCommitBody) == *"[package-all-types]"* ]] || [[ "${PACKAGE_ALL_TYPES}" == true ]] || common_isReleaseOrPreReleaseOrMilestoneOrMasterBranch "$branch"; then
    export INCLUDE_ALL_INSTALLER_TYPES="true"
  else
    export INCLUDE_ALL_INSTALLER_TYPES="false"
  fi
}

setProfileType(){
  local disableDEB="!deb"
  local disableRPM="!rpm"
  local disableDocker="!docker"
  local disableStandalone="!standalone"

  case "${JFROG_CLI_BUILD_NAME}" in
      *"installers-docker"* )
          ## Build artifacts under docker profile
          export INSTALLER_OSS_PROFILE_TYPE="docker"
          export PRODUCT_DISABLE_PROFILES="-pl ${disableDEB},${disableRPM},${disableStandalone}"
          ;;
      *"installers-rpm"* )
          ## Build artifacts under rpm profile
          export INSTALLER_OSS_PROFILE_TYPE="rpm"
          export PRODUCT_DISABLE_PROFILES="-pl ${disableDEB},${disableDocker},${disableStandalone}"
          ;;
      *"installers-deb"* )
          ## Build artifacts under deb profile
          export INSTALLER_OSS_PROFILE_TYPE="deb"
          export PRODUCT_DISABLE_PROFILES="-pl ${disableDocker},${disableRPM},${disableStandalone}"
          ;;
      * )
          ## Build artifacts under release profile - includes standalone
          export INSTALLER_OSS_PROFILE_TYPE="release"
          export PRODUCT_DISABLE_PROFILES="-pl ${disableDEB},${disableRPM},${disableDocker}"
          ;;
  esac
}