/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.event;

import javax.annotation.Nonnull;
import java.util.List;
import java.util.Set;
import java.util.stream.Stream;

/**
 * A business service interface to interact with the tree events.
 *
 * @author Yossi Shaul
 */
public interface EventsService {
    /**
     * @return True if node events recording is enabled
     */
    boolean isEnabled();

    /**
     * @return Count of all events in the log
     */
    long getEventsCount();

    /**
     * List of ordered events to add
     *
     * @param events List of ordered events to create
     */
    void appendEvents(List<EventInfo> events);

    /**
     * @param eventId    The eventId to get events after
     * @param repoKey    The repository key of events to get
     * @param fileSuffix The suffix of the file if events to get
     * @param limit      Limit the number of events receive (0 means no limit)
     * @return Ordered list of events happened since the input eventId (not inclusive) on the repo key
     */
    List<EventInfo> getEventsAfter(long eventId, String repoKey, String fileSuffix, int limit);

    default List<EventInfo> getEventsAfter(long eventId, String repoKey) {
        return getEventsAfter(eventId, repoKey, null, 0);
    }

    /**
     * count events after the specified time exclusive
     * @param eventId The timestamp from to get events after
     * @param repoKey       The repository key of events to get
     * @param fileSuffix    The suffix of the file if events to get
     * @return count of events happened since the input eventId (not inclusive) on the repo key
     */
    int countEventsAfter(long eventId, String repoKey, String fileSuffix);


    @Deprecated
    List<EventInfo> getEventsByTimestamp(long timestamp);

    /**
     * @param timestamp The timestamp to get events since
     * @param repoKey   The repository key or events to get
     * @return Ordered list of events happened since the input timestamp on the repo key. Including events happened at
     * the same time as the input timestamp
     */
    @Deprecated
    List<EventInfo> getEventsAfterInclusive(long timestamp, String repoKey);

    @Deprecated
    default List<EventInfo> getEventsAfterInclusive(long timestamp) {
        return getEventsAfterInclusive(timestamp, null);
    }

    /**
     * @return The timestamp in millis of the first recorded event
     */
    @Deprecated
    long getFirstEventTimestamp();

    /**
     * @return The timestamp in millis of the last recorded event
     */
    @Deprecated
    long getLastEventLogTimestamp();

    /**
     * @return The id of the first recorded event
     */
    long getFirstEventLogId(String repoKey);

    /**
     * @return The id of the last recorded event
     */
    long getLastEventLogId();

    /**
     * fetch all events created after timestamp that correspond to this repoKey
     *
     * @param timestamp - events since this time
     * @param maxEvents - maxEvents to fetch
     * @return a stream of events objects
     */
    @Deprecated
    Stream<EventInfo> getEventsSince(long timestamp, long maxEvents);

    /**
     * Get all the events newer from the from and older than to
     *
     * @param from Beginning of the interval non-inclusive timestamp in millis
     * @param to   End of the interval non-inclusive timestamp in millis
     * @return Events newer than the given from and older then the to ordered by timestamp
     */
    @Deprecated
    List<EventInfo> getEventsInterval(long from, long to, int maxEvents, String repoKey);

    // TODO: 19/07/2020 startInclusive,endExclusive should be replaced to eventId instead of timestamp
    void deleteRange(long startInclusive, long endExclusive);

    /**
     * Shift events from node_events_tmp to the final table node_events adding sequential event_id
     *
     * @return effected repositories
     */
    @Nonnull
    Set<String> shiftEvents();

    /**
     * count events in range by repokey
     *
     * @param startFrom the starting range inclusive
     * @param end the end timestamp non inclusive
     * @param repoKey the events related to the repo
     */

    int countEventsRangeByRepoKey(long startFrom, long end, @Nonnull String repoKey);
}
