/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.onboarding;

import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.artifactory.addon.AddonsManager;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.entitlement.EntitlementType;
import org.artifactory.projects.ProjectsService;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.ui.rest.model.onboarding.CreateQuickReposModel;
import org.artifactory.ui.rest.service.admin.configuration.repositories.CreateRepositoryConfigService;
import org.artifactory.ui.utils.DefaultRepoCreator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

/**
 * Service for creating default repositories for repo types stated in CreateDefaultReposModel
 *
 * @author nadavy
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
@Slf4j
public class CreateQuickReposService implements RestService<CreateQuickReposModel> {

    @Autowired
    private CreateRepositoryConfigService createRepoService;

    @Autowired
    private AddonsManager addonsManager;

    @Autowired
    ProjectsService projectsService;

    private final DefaultRepoCreator defaultRepoCreator = new DefaultRepoCreator();

    @Override
    public void execute(ArtifactoryRestRequest<CreateQuickReposModel> request, RestResponse response) {
        CreateQuickReposModel reposToCreate = request.getImodel();
        createDefaultRepositoriesFromModel(response, reposToCreate);
    }

    /**
     * Create default repositories as defined in defaultRepository.json
     */
    private void createDefaultRepositoriesFromModel(RestResponse response, CreateQuickReposModel reposToCreate) {
        CreateDefaultReposResponseModel createDefaultReposResponseModel = new CreateDefaultReposResponseModel();
        ArtifactoryContext artifactoryContext = ContextHelper.get();
        boolean isSupportRemoteRepositories = !addonsManager.isEntitled(EntitlementType.EDGE);
        reposToCreate.getRepositories().forEach(model -> projectsService
                .validateRepoConfigIsMatchingToProjectSpec(model.getProject(), model.getRepoName(), model.getEnvs()));
        reposToCreate.getRepositories()
                .forEach(repoModel -> defaultRepoCreator.createQuickRepos(createRepoService, response, repoModel,
                        createDefaultReposResponseModel,isSupportRemoteRepositories));
        if (createDefaultReposResponseModel.isValid()) {
            handleValidRepoCreation(artifactoryContext);
            response.iModel(createDefaultReposResponseModel).responseCode(HttpStatus.SC_CREATED);
        } else {
            response.responseCode(HttpStatus.SC_BAD_REQUEST);
        }
    }

    /**
     * Handles valid repo creation from REST UI -
     * save and reload config descriptor
     * and export settings to yaml (if from onboarding)
     */
    private void handleValidRepoCreation(ArtifactoryContext artifactoryContext) {
        artifactoryContext.getCentralConfig()
                .saveEditedDescriptorAndReload(defaultRepoCreator.getConfigDescriptor());
    }
}
